package ti.myservice;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;
import android.support.v4.app.NotificationCompat;
import org.appcelerator.kroll.common.Log;
import org.appcelerator.titanium.TiApplication;


public final class ForegroundService extends Service
{
	/** The tag name to be used by the Log() method. Will prefix log message with this tag name. */
	private static final String TAG = "ForegroundService";

	/** Stores a reference to this service if running. Set to null if service never started or not currently running. */
	private static ForegroundService instance;

	/** Binder object providing access to this service via the onBind() method call. Created dynamically. */
	private ForegroundService.Binder binder;


	/** Called when the Android OS creates this service. */
	@Override
	public void onCreate()
	{
		Log.i(TAG, "onCreate() called.");

		// Keep a reference to this service instance to be made available via the static getInstance() method.
		ForegroundService.instance = this;

		// Put this service into the foreground.
		try {
//TODO: *** Configure the notification here. ***
			// Set up the foreground service's status bar notification.
			final int NOTIFICATION_ID = 1234;	// <- Set this to a unique ID.
			final String NOTIFICATION_CHANNEL_NAME = "service_channel";
			if (Build.VERSION.SDK_INT >= 26) {
				NotificationChannel channel = new NotificationChannel(
						NOTIFICATION_CHANNEL_NAME, "Service Notifications",
						NotificationManager.IMPORTANCE_LOW);
				NotificationManager manager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
				manager.createNotificationChannel(channel);
			}
			NotificationCompat.Builder notificationBuilder;
			notificationBuilder = new NotificationCompat.Builder(this, NOTIFICATION_CHANNEL_NAME);
			notificationBuilder.setSmallIcon(android.R.drawable.stat_sys_warning);
			notificationBuilder.setContentTitle(TiApplication.getInstance().getAppInfo().getName());
			notificationBuilder.setContentText("Service is running.");

			// Set up an intent to launch/resume the app when the above notification is tapped on.
			Intent launchIntent = getPackageManager().getLaunchIntentForPackage(getPackageName());
			launchIntent.setPackage(null);
			launchIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			launchIntent.addFlags(Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);
			notificationBuilder.setContentIntent(PendingIntent.getActivity(this, 0, launchIntent, 0));

			// Enable this service's foreground state.
			startForeground(NOTIFICATION_ID, notificationBuilder.build());
		} catch (Exception ex) {
			Log.e(TAG, "Failed to put service into the foreground.", ex);
		}

//TODO: *** Put your code "here" now that the service has started. ***
	}

	/** Called when the Android OS is about to destroy this service. */
	@Override
	public void onDestroy()
	{
		Log.i(TAG, "onDestroy() called.");

		// Remove reference to this service now that it's destroyed. This means static getInstance() will return null.
		ForegroundService.instance = null;

//TODO: *** The service has stopped. Do your code cleanup here. ***
	}

	/**
	 * Called when the service has just been started.
	 * @param intent The intent used to start this service. Can be null.
	 * @param flags Provides additional flags such as START_FLAG_REDELIVERY, START START_FLAG_RETRY, etc.
	 * @param startId Unique integer ID to be used by stopSelfResult() method, if needed.
	 * @return Returns a "START_*" constant indicating how the Android OS should handle the started service.
	 */
	@Override
	public int onStartCommand(Intent intent, int flags, int startId)
	{
		return Service.START_STICKY;
	}

	/**
	 * Called when a client of this service wants direct access to this service via a binder object.
	 * @param intent The intent that was used to bind this service.
	 * @return Returns a binder object used to access this service.
	 */
	@Override
	public IBinder onBind(Intent intent)
	{
		if (this.binder == null) {
			this.binder = new ForegroundService.Binder(this);
		}
		return this.binder;
	}

	/**
	 * Starts this service, if not already running.
	 * @return
	 * Returns true if service was started or already running.
	 * Returns false if a startup error occurred which can happen if "FOREGROUND_SERVICE" permission is missing.
	 */
	public static boolean start()
	{
		// Do not continue if this service is already running.
		if (ForegroundService.instance != null) {
			return true;
		}

		// Start the service.
		boolean wasStarted = false;
		try {
			TiApplication context = TiApplication.getInstance();
			Intent serviceIntent = new Intent(context, ForegroundService.class);
			if (Build.VERSION.SDK_INT >= 26) {
				context.startForegroundService(serviceIntent);
			} else {
				context.startService(serviceIntent);
			}
			wasStarted = true;
		} catch (Exception ex) {
			Log.e(TAG, "Failed to start service.", ex);
		}
		return wasStarted;
	}

	/** Stop this service, if currently running. */
	public static void stop()
	{
		// Fetch the service instance.
		ForegroundService thisService = ForegroundService.instance;
		if (thisService == null) {
			return;
		}

		// Stop this service.
		try {
			thisService.stopSelf();
		} catch (Exception ex) {
			Log.e(TAG, "Failed to stop service.", ex);
		}
	}

	/**
	 * Gets a reference to the one and only ForegroundService instance created by the Android OS, if running.
	 * @return Returns a reference to this service if running. Returns null if not running.
	 */
	public static ForegroundService getInstance()
	{
		return ForegroundService.instance;
	}


	/** Binder providing external access to the ForegroundService object. */
	private static class Binder extends android.os.Binder
	{
		/** Reference to the service this binder provides access to. */
		private ForegroundService service;

		/**
		 * Creates a new binder providing access to the given service.
		 * @param service Reference to the service this binder will provide access to.
		 */
		public Binder(ForegroundService service)
		{
			this.service = service;
		}

		/**
		 * Gets the service this binder provides access to.
		 * @return Returns the service this binder provides access to.
		 */
		Service getService()
		{
			return this.service;
		}
	}
}
