#!/bin/bash

# Print build script usage if "--help" command line argument provided as 1st argument.
if [ "$1" = "-h" ] || [ "$1" = "--help" ]
then
	echo "build [TitaniumProjectPath]"
	echo "- TitaniumProjectPath:  Path to Titanium project to unzip module to. (optional)"
	exit 0
fi

# Fetch the directory path this build script is in.
buildScriptPath=$(dirname "$0")

# Fetch the current working directory path. (Might not be the same path as this build script.)
workingDirectoryPath=$(pwd)

# Fetch optional 1st argument.
# If set, then it's expected to reference a Titanium project path to unzip built module to.
projectPath="$1"
if [ ! -z "$projectPath"  ] && [ ! -d "$projectPath" ]
then
	echo "Error: Argument does not reference a Titanium project directory path."
	exit 1
fi

# Attempt to build the module.
cd "$buildScriptPath/android"
if [ -d "dist" ]
then
	rm -rf "dist"
fi
appc run -p android --build-only
if [ $? -ne 0 ]
then
	exit $?
fi

# Do not continue if not given a Titanium project path argument.
# The caller only wants the module built and does not want to unzip it to an app project.
if [ -z "$projectPath"  ]
then
	exit 0
fi

# Fetch an absolute path to the built module.
zipFilePath=$(pwd)/$(find ./dist -name "*.zip" -print -quit)

# Unzip the built module to the given Titanium project directory.
cd "$workingDirectoryPath"
cd "$projectPath"
if [ ! -d "modules" ]
then
	mkdir "modules"
fi
if [ -d "modules/android" ]
then
	rm -rf "modules/android"
fi
unzip -o "$zipFilePath" -d .
exit $?
